<?php

namespace Drupal\string;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Plugin\Discovery\ContainerDerivativeDiscoveryDecorator;
use Drupal\Core\Plugin\Discovery\YamlDiscovery;

/**
 * StringManager class definition.
 */
class StringGroupManager extends DefaultPluginManager {

  const TITLE = 'title';
  const PARENT = 'parent';

  const DEFAULT_GROUP_ID = 'string.default.group';

  /**
   * Constructs a new BananaDashboardManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   */
  public function __construct(\Traversable $namespaces, ModuleHandlerInterface $module_handler, CacheBackendInterface $cache_backend) {
    // Add more services as required.
    $this->moduleHandler = $module_handler;
    $this->setCacheBackend($cache_backend, 'string_group', ['string_group']);
    parent::__construct(FALSE, $namespaces, $module_handler);
  }

  /**
   * {@inheritdoc}
   */
  protected function getDiscovery() {
    if ($this->discovery === NULL) {
      $this->discovery = new YamlDiscovery('string_group', $this->moduleHandler->getModuleDirectories());
      // $this->discovery->addTranslatableProperty('string', 'string_context');
      $this->discovery = new ContainerDerivativeDiscoveryDecorator($this->discovery);
    }
    return $this->discovery;
  }

  /**
   * {@inheritdoc}
   *
   * @phpstan-ignore missingType.parameter,missingType.parameter
   */
  public function processDefinition(&$definition, $plugin_id): void {
    parent::processDefinition($definition, $plugin_id);
    if (empty($definition['id'])) {
      throw new PluginException('Plugin property "id" is required for string plugin definition.');
    }
    if (empty($definition[self::TITLE])) {
      throw new PluginException(sprintf('Plugin property "%s" is required for string plugin definitions.', self::TITLE));
    }
  }

  /**
   * Finds plugin definitions.
   *
   * @return array<string, mixed>
   *   List of definitions to store in cache.
   */
  protected function findDefinitions(): array {
    $definitions = $this->getDiscovery()->getDefinitions();
    foreach ($definitions as $plugin_id => &$definition) {
      $this->processDefinition($definition, $plugin_id);
    }
    $this->alterDefinitions($definitions);
    // If this plugin was provided by a module that does not exist, remove the
    // plugin definition.
    foreach ($definitions as $plugin_id => $plugin_definition) {
      $provider = $this->extractProviderFromDefinition($plugin_definition);
      if ($provider && !in_array($provider, ['core', 'component']) && !$this->providerExists($provider)) {
        unset($definitions[$plugin_id]);
      }
    }
    return $definitions;
  }

}
